from django.conf import settings
from django.template import Template, Context
from os import makedirs, path
from .models import ActiveShiny


def generate_shiny(hash, shiny_context):
	"""populate the shiny app"""
	makedirs(path.join(settings.SHINY_SERVER_DIRECTORY,hash))
	#context from function overwrites any in settings file
	if shiny_context:
		context = Context(shiny_context)
	else:
		context = Context(settings.SHINY_CONTEXT)
	
	if type(settings.SHINY_TEMPLATE_FILE) == type(str()):
		with open(settings.SHINY_TEMPLATE_FILE, 'r') as file:
			shiny_template = Template(file.read())
		with open(path.join(settings.SHINY_SERVER_DIRECTORY , hash , "shiny.R"), 'w') as shiny_application:
			shiny_application.write(shiny_template.render(context))
	else:
		#if SHINY_TEMPLATE_FILE is list of files then SHINY_TEMPLATE_DIRECTORY should be defined
		for shiny_file in settings.SHINY_TEMPLATE_FILE:
			with open(path.join(settings.SHINY_TEMPLATE_DIRECTORY,shiny_file), 'r') as file:
				shiny_template = Template(file.read())
			with open(path.join(settings.SHINY_SERVER_DIRECTORY , hash , shiny_file), 'w') as shiny_application:
				shiny_application.write(shiny_template.render(context))




	
	
	



from django.conf import settings
from django.shortcuts import render
from django.http import Http404, JsonResponse
from django.utils import timezone
from .models import ShinyUserHash, ActiveShiny
from .utils import  generate_shiny
from secrets import token_hex
from datetime import timedelta
from importlib.machinery import SourceFileLoader


def hash_view(request, id):
	try:
		shinyuser = ShinyUserHash.objects.get(user__id=id)
	except ShinyUserHash.DoesNotExist:
		raise Http404("Not Found")
	hash = shinyuser.user_hash
	try:
		context_module = SourceFileLoader("context",settings.SHINY_CONTEXT_MODULE).load_module()
		shiny_context = context_module.shiny_context(request)
	except Exception as e: #setting not configured / file not found / function improperly named
		print(e)
		shiny_context = {}
	generate_shiny(hash, shiny_context)
	shinyuser.user_hash = token_hex(16)
	shinyuser.save()
	template_name = 'serve_shiny/shiny_wrapper.html'
	context = {'hash': hash, "shiny_url": settings.SHINY_SERVER_URL}
	return render(request, template_name, context)

def update_connection(request):
	if request.method != 'POST':
		return HttpResponseNotFound("NOT FOUND")
	#post data coming in strange from javascript
	hash = request.POST.get(" name")[10:42]
	if hash is None or hash == "":
		return HttpResponseNotFound("Not Found")
	now = timezone.now()
	allowance = timedelta(minutes=10)
	try:
		is_active = ActiveShiny.objects.get(project_dir = hash)
	except ActiveShiny.DoesNotExist:
		is_active = False
	if is_active:
		is_active.expiration = now + allowance
		is_active.save(update_fields = ['expiration'])
	else:
		activate = ActiveShiny(project_dir = hash, expiration = (now + allowance) )
		activate.save() 
	return JsonResponse({'status': 'updated'})




from django.db.models.signals import post_save
from django.contrib.auth.models import User
from django.dispatch import receiver
from .models import ShinyUserHash
from secrets import token_hex

@receiver(post_save, sender=User)
def create_hash(sender, instance, created, **kwargs):
    if created:
        hash = token_hex(16)
        ShinyUserHash.objects.create(user=instance, user_hash=hash)


@receiver(post_save, sender=User)
def save_hash(sender, instance, created, **kwargs):
    instance.shinyuserhash.save()



from django.db import models
from django.contrib.auth.models import User



class ShinyUserHash(models.Model):
    user_hash = models.CharField(max_length=32)
    user = models.OneToOneField(User, on_delete=models.CASCADE)

class ActiveShiny(models.Model):
    project_dir = models.CharField(max_length = 32)
    expiration = models.DateTimeField()
	
    def __str__(self):
        return "Active Shiny Session {} expires {}".format(self.project_dir, self.expiration)


import django
from django.conf import settings
from importlib.machinery import SourceFileLoader
from django.utils import timezone
from os import makedirs
from pathlib import Path
from shutil import rmtree
import sys




def clean_up_shiny():
	"""Remove all apps in shiny directory that are expired."""
	print("in cleanup func")
	expired_sessions = ActiveShiny.objects.filter(expiration__lt = timezone.now())
	for session in expired_sessions:
		session_dir = Path(settings.SHINY_SERVER_DIRECTORY, session.project_dir)
		if session_dir.exists() and session_dir.is_dir():
			rmtree(session_dir)
		session.delete()

if __name__ == "__main__":
	user_settings = SourceFileLoader("settings", sys.argv[1]).load_module()
	settings.configure(DATABASES=user_settings.DATABASES,
		SHINY_SERVER_DIRECTORY=user_settings.SHINY_SERVER_DIRECTORY,
		INSTALLED_APPS=user_settings.INSTALLED_APPS,
		TIME_ZONE=user_settings.TIME_ZONE,
		USE_TZ=user_settings.USE_TZ)
	django.setup()
	from serve_shiny.models import ActiveShiny
	clean_up_shiny()
	



from django.urls import path, include

from .views import hash_view, update_connection

urlpatterns = [
	path('hash/<int:id>/' , hash_view, name ='hash'),
	path('connect/' , update_connection, name = 'connection'), 
	]

